// Forlix SoBall
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs

#define COMMON_PREFIX               "prop_"
#define COMMON_PREFIX_LEN           5

#define PHYS_CLASSNAME              "prop_physics"
#define PHYS_CLASSNAME_BARE         "physics"
#define PHYS_CLASSNAME_POSTFIX      "prop_physics_"

#define PHYS_CLASSNAME_BARE_LEN     7
#define PHYS_CLASSNAME_POSTFIX_LEN  13

#define SOCC_CLASSNAME              "prop_soball"
#define SOCC_CLASSNAME_BARE         "soball"
#define SOCC_CLASSNAME_POSTFIX      "prop_soball_"

#define SOCC_CLASSNAME_BARE_LEN     6
#define SOCC_CLASSNAME_POSTFIX_LEN  12


public Event_PlayerSpawn(Handle:event,
                         const String:Event_type[],
                         bool:dontBroadcast)
{
  new client = GetClientOfUserId(GetEventInt(event, "userid"));

  if(!GetClientTeam(client))
  // not an actual spawn (just "entered the game")
    return;

  if(!p_spawned[client])
  // players first spawn since connect
  {
    p_spawned[client] = true;

    if(announcement_type == 1)
      PrintAnnouncementMsg(client);
  }

  return;
}


public Event_RoundStart(Handle:event,
                        const String:name[],
                        bool:dontBroadcast)
{
  round_freeze_end = -1.0;
  SetupEntities();

  // reset all spawn counts, no matter if clients are ingame
  for(new client = 1; client <= MaxClients; client++)
    p_ballspawns[client] = 0;

  if(spawncount_per_round
  && announcement_type == 0)
    PrintAnnouncementMsg(0);

  return;
}


public Event_RoundFreezeEnd(Handle:event,
                            const String:Event_type[],
                            bool:dontBroadcast)
{
  round_freeze_end = GetTickedTime();
  return;
}


public Event_PropShot(const String:name[],
                      caller,
                      activator,
                      Float:delay)
{
  decl String:classname[32];
  decl String:classname_postfix[32];

  decl String:targetname[32];
  GetEntPropString(caller, Prop_Data, "m_iName", targetname, sizeof(targetname));

  if(!strcmp(targetname, SOCC_CLASSNAME))
  // if targetname(!) equals SOCC_CLASSNAME, its one of our balls.
  // we can't check against the classname here, because after spawn its generic
    classname_postfix = SOCC_CLASSNAME_POSTFIX;
  else
  // a random prop_physics - we still want to attach the "shooter's" userid
    classname_postfix = PHYS_CLASSNAME_POSTFIX;

  new shooter_userid = 0;

  if(activator > 0
  && activator <= MaxClients
  && IsClientInGame(activator))
    shooter_userid = GetClientUserId(activator);

  // finally, set the classname in form of "prop_soball_12345_1234567" or "prop_physics_12345_1234567"
  Format(classname, sizeof(classname), "%s%u_%.f", classname_postfix, shooter_userid, GetTickedTime());
  DispatchKeyValue(caller, "classname", classname);

  return;
}


public Action:Event_PlayerDeath(Handle:event,
                                const String:name[],
                                bool:dontBroadcast)
{
  decl String:weapon[32];
  GetEventString(event, "weapon", weapon, sizeof(weapon));

  if(strncmp(weapon, COMMON_PREFIX, COMMON_PREFIX_LEN))
  // classname doesn't start with COMMON_PREFIX (eg. "prop_")
    return(Plugin_Continue);

  new attacker_userid = 0;
  new shot_time = 0;

  if(!strncmp(weapon[COMMON_PREFIX_LEN], SOCC_CLASSNAME_BARE, SOCC_CLASSNAME_BARE_LEN)
  && weapon[SOCC_CLASSNAME_POSTFIX_LEN-1] == '_')
  // classname is SOCC_CLASSNAME_POSTFIX (eg. "prop_soball_")
  {
    SetEventString(event, "weapon", SOCC_CLASSNAME);
    new uid_len = FindCharInString(weapon[SOCC_CLASSNAME_POSTFIX_LEN], '_');

    if(uid_len <= 0)
      return(Plugin_Continue);

    weapon[SOCC_CLASSNAME_POSTFIX_LEN+uid_len] = '\0';

    attacker_userid = StringToInt(weapon[SOCC_CLASSNAME_POSTFIX_LEN]);
    shot_time = StringToInt(weapon[SOCC_CLASSNAME_POSTFIX_LEN+uid_len+1]);
  }
  else
  if(!strncmp(weapon[COMMON_PREFIX_LEN], PHYS_CLASSNAME_BARE, PHYS_CLASSNAME_BARE_LEN)
  && weapon[PHYS_CLASSNAME_POSTFIX_LEN-1] == '_')
  // classname is PHYS_CLASSNAME_POSTFIX (eg. "prop_physics_")
  {
    SetEventString(event, "weapon", PHYS_CLASSNAME);
    new uid_len = FindCharInString(weapon[PHYS_CLASSNAME_POSTFIX_LEN], '_');

    if(uid_len <= 0)
      return(Plugin_Continue);

    weapon[PHYS_CLASSNAME_POSTFIX_LEN+uid_len] = '\0';

    attacker_userid = StringToInt(weapon[PHYS_CLASSNAME_POSTFIX_LEN]);
    shot_time = StringToInt(weapon[PHYS_CLASSNAME_POSTFIX_LEN+uid_len+1]);
  }

  if(!attacker_userid
  || (kill_crediting_time
  && (GetTickedTime()-shot_time > kill_crediting_time)))
  // shooter was not a player
  // or weapon was neither SOCC_CLASSNAME nor PHYS_CLASSNAME
  // or death occured more than kill_crediting_time secs after shot
    return(Plugin_Continue);

  new victim_userid = GetEventInt(event, "userid");

  if(attacker_userid != victim_userid)
  // no suicide, so we have to set some scores
  {
    new attacker = GetClientOfUserId(attacker_userid);
    new victim = GetClientOfUserId(victim_userid);

    if(attacker)
    {
      // decide if ordinary or team kill
      if(GetClientTeam(attacker) != GetClientTeam(victim))
        SetEntProp(attacker, Prop_Data, "m_iFrags", GetClientFrags(attacker)+1);
      else
        SetEntProp(attacker, Prop_Data, "m_iFrags", GetClientFrags(attacker)-1);

      AcceptEntityInput(game_score_ref, "ApplyScore", attacker, game_score_ref);

      // observertarget to killer - also adds freezecam support
      SetEntPropEnt(victim, Prop_Send, "m_hObserverTarget", attacker);
    }

    // victim may not lose frags, just get a death point!
    SetEntProp(victim, Prop_Data, "m_iFrags", GetClientFrags(victim)+1);
  }

  SetEventInt(event, "attacker", attacker_userid);
  return(Plugin_Continue);
}
